/*
***********************************************************************
China Mine version 1.1
Copyright (C) 1998, 1999, 2000  Ma Guokai, from People's Republic of China

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If what you get is only part of the program, send an email to akey@fudan.edu
give the name of the program and the version number (newest by default), I'll
send you a copy of the full package.
*****************************************************************************
*/

import java.applet.Applet;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

class MainCanvas extends Canvas implements Runnable, MouseListener, KeyListener,
												MouseMotionListener, FocusListener
{
	private int faceState=Constant.SMILE;
	private int[][] blockState=new int[Constant.maxMatrixWidth][Constant.maxMatrixHeight];
	private int[][] blockContain=new int[Constant.maxMatrixWidth][Constant.maxMatrixHeight];
	private boolean firstUncover,blockEnabled=true;
	public Graphics g;
	private int mineCount;
	public Image bkImage;
	public Graphics bkGraphics;

	private int mouseX,mouseY;
	public boolean leftDown=false,rightDown=false,insideFace=false,moveSee=true, tagged=false;
	private boolean alive=true;;

	public MainCanvas()
	{
		addMouseListener(this);
		addKeyListener(this);
		addMouseMotionListener(this);
		addFocusListener(this);
	}

	private boolean started=false;
	private LevelConst lc;
	public StyleConst sc;
	private boolean firstInit=true;
	public void init()
	{
		lc=Constant.levels[Config.level];
		sc=Constant.styles[Config.style];
		if(firstInit)
		{
			Constant.screenDim=Toolkit.getDefaultToolkit().getScreenSize();
			setSize(Constant.screenDim);
			firstInit=false;
		}
		requestFocus();

		bkImage=createImage(lc.panelW+edgeSize*2, lc.panelH+edgeSize*2);
		bkGraphics=bkImage.getGraphics();
		this.g=getGraphics();
	}

	public void start()
	{
		lc=Constant.levels[Config.level];
		restart();
		started=true;
	}

	public void update(Graphics g)
	{
		if(started)
			paintCanvas();
	}

	public void paint(Graphics g)
	{
		if(started)
			paintCanvas();
	}

	private boolean mineStatusChanged=true;
	public boolean styleChanged=true;
	private void paintCanvas()
	{
		if(mineStatusChanged || styleChanged)
		{
			paintPanel();
			paintFace(true);
			paintTime();
			paintMine();
			for(int i=0;i<lc.rows;i++)
				for(int j=0;j<lc.cols;j++)
					paintBlock(j,i,false);
			styleChanged=false;
		}
		fresh(true);
	}

	private static int canvasX=0, canvasY=0, edgeSize=10;
	private void fresh(boolean need)
	{
		if(g!=null && need)
		{
			if(Constant.isCombating)
			{
				Dimension tempDim=getSize();
				canvasX=(tempDim.width-lc.panelW)/2;
				canvasY=(tempDim.height-lc.panelH)/2;
			}
			else
			{
				canvasX=canvasY=0;
			}
			g.drawImage(bkImage,canvasX-edgeSize,canvasY-edgeSize,this);
			mineStatusChanged=false;
		}
	}

	private void paintPanel()
	{
		if(bkGraphics!=null)
		{
			drawButton(bkGraphics, 0, 0, lc.panelW, lc.panelH, sc.bolderBig,
					Color.lightGray, Color.white, Color.gray);
			drawButton(bkGraphics, Constant.blockX-3, Constant.mineY-7,
					lc.cols*Constant.blockW+6, Constant.LEDH+14, sc.bolderUp,
					Color.lightGray, Color.gray, Color.white);
			drawButton(bkGraphics, Constant.blockX-3, Constant.blockY-3,
					lc.cols*Constant.blockW+6, lc.rows*Constant.blockH+6, sc.bolderDown,
					Color.lightGray, Color.gray, Color.white);
			drawButton(bkGraphics, Constant.mineX-1,Constant.mineY-1, 3*Constant.LEDW+2, Constant.LEDH+2, sc.bolderLED,
					Color.lightGray, Color.gray, Color.white);
			drawButton(bkGraphics, lc.timeX-1, Constant.timeY-1, 3*Constant.LEDW+2, Constant.LEDH+2, sc.bolderLED,
					Color.lightGray, Color.gray, Color.white);
		}
	}
	
	private void paintBlock(int col,int row,boolean drawBlank)
	{
		if(bkGraphics!=null)
			bkGraphics.drawImage(sc.block[drawBlank?Constant.NONE:blockState[col][row]],
				Constant.blockX+Constant.blockW*col+edgeSize,
				Constant.blockY+Constant.blockH*row+edgeSize,this);
	}

	private void paintFace(boolean isUp)
	{
		if(bkGraphics!=null)
			bkGraphics.drawImage(sc.face[isUp?faceState:Constant.DOWN], lc.faceX+edgeSize, Constant.faceY+edgeSize,this);
	}

	private void paintLED(int digit,int x,int y)
	{
		if(bkGraphics!=null)
			bkGraphics.drawImage(sc.led[digit],x+edgeSize,y+edgeSize,this);
	}

	private void paintLED3(int count,int x,int y)
	{
		if(count>=0)
			for(int i=0;i<3;i++)
			{
				paintLED(count%10,x+Constant.LEDW*(2-i),y);
				count=count/10;
			}
		else
		{
			count=-count;
			paintLED(10,x,y);
			for(int i=0;i<2;i++)
			{
				paintLED(count%10,x+Constant.LEDW*(2-i),y);
				count=count/10;
			}
		}
	}

/*mineLED start*/
private int mineCount1;
	private void initMine()
	{
		mineCount1=lc.mines;
		paintMine();
	}

	private void incMine()
	{
		mineCount1++;
		paintMine();
	}

	private void decMine()
	{
		mineCount1--;
		paintMine();
	}

	private void paintMine()
	{
		paintLED3(mineCount1,Constant.mineX,Constant.mineY);
	}
/*mineLED end*/

/*timeLED start*/
private int timeCount,timeCount1;
private boolean firstTime=true,timeStarted=false;
private Thread timeThread;
private long absTime,tempLong;
private int timerRate=10;

	private void initTime()
	{
		stopTime();
		timeCount=0;
		timeCount1=0;
		paintTime();
	}

	private void startTime()
	{
		if(timeStarted)
			return;
		timeStarted=true;
		timeCount=1;
		timeCount1=timeCount*timerRate;
		absTime=System.currentTimeMillis();
		paintTime();
		fresh(true);
		firstTime=true;
		timeThread=new Thread(this);
		timeThread.start();
	}

	private void stopTime()
	{
		timeStarted=false;
		if(timeThread!=null)
		{
			timeThread.stop();
			timeThread=null;
		}
	}

	private void paintTime()
	{
		timeCount1=(timeCount>timeCount1/timerRate)?(timeCount*timerRate):timeCount1;
		paintLED3( timeCount1/timerRate, lc.timeX, Constant.timeY);
	}

	public void run()
	{
		while (timeThread!=null)
		{
			try
			{
				if(firstTime)
				{
					Thread.sleep((int)( 1000*Math.random() ));
					firstTime=false;
					if( (System.currentTimeMillis()-absTime)/1000>1 )
					{
						tempLong=1+(System.currentTimeMillis()-absTime)/1000;
						timeCount=(tempLong>Constant.anonymousTime)?Constant.anonymousTime:(int)tempLong;
					}
					else
						timeCount=2;
					absTime=System.currentTimeMillis()-(timeCount-2)*1000;
					timeCount1=timeCount*timerRate;
					paintTime();
					fresh(true);
					continue;
				}
				else
				{
					Thread.sleep(1000/timerRate);
					if(timeCount1<Constant.anonymousTime*timerRate)
						timeCount1++;
					else
						timeCount1=Constant.anonymousTime*timerRate;
					if(timeCount<Constant.anonymousTime && timeCount!=2+(System.currentTimeMillis()-absTime)/1000)
					{
						tempLong=2+(System.currentTimeMillis()-absTime)/1000;
						timeCount=(tempLong>Constant.anonymousTime)?Constant.anonymousTime:(int)tempLong;
						paintTime();
						fresh(true);
						continue;
					}
					if(timeCount>Constant.anonymousTime)
						timeCount=Constant.anonymousTime;
				}
			}catch(InterruptedException e){}
		}
	}

/*timeLED end*/
	public void restart()
	{
		leftDown=false;
		rightDown=false;
		insideFace=false;
		moveSee=true;
		tagged=false;
		alive=true;
		for(int i=0;i<lc.rows;i++)
			for(int j=0;j<lc.cols;j++)
				blockState[j][i]=Constant.UNTOUCHED;
		blockEnabled=true;
		firstUncover=true;
		mineCount=lc.cols*lc.rows-lc.mines;
		initMine();
		initTime();
		faceState=Constant.SMILE;
		mineStatusChanged=true;
		paintCanvas();
	}

	private void die(int col,int row)
	{
		faceState=Constant.AH;
		alive=false;
		stopTime();
		paintFace(true);
		for(int j=0;j<lc.rows;j++)
			for(int i=0;i<lc.cols;i++)
			{
				int contain=blockContain[i][j],
					state=blockState[i][j];
				if(i==col && j==row)
					blockContain[i][j]=blockState[i][j]=Constant.BAN;
				else if(contain==Constant.MINE && state!=Constant.FLAG)
					blockState[i][j]=Constant.MINE;
				else if(contain!=Constant.MINE && state==Constant.FLAG)
					blockState[i][j]=Constant.WRONG;
				else
					continue;

				paintBlock(i,j,false);
			}
		blockEnabled=false;
	}

	private void win()
	{
		int timeCount=this.timeCount,
			timeCounter1=this.timeCount1,
			level=Config.level;
		boolean tagged=this.tagged;
		faceState=Constant.HAPPY;
		paintFace(true);
		for(int j=0;j<lc.rows;j++)
			for(int i=0;i<lc.cols;i++)
			{
				if(blockContain[i][j]==Constant.MINE)
					tag(i,j,false);
			}
		setEnabled(true);
		leftDown=false;
		rightDown=false;
		insideFace=false;
		moveSee=true;
		blockEnabled=false;
		regist((timeCount>timeCount1/timerRate)?timeCount:(timeCount1/timerRate),level,tagged);
	}

	private void tag(int col,int row,boolean flip)
	{
		if(!isAblock(col,row))
			return;
		int state=blockState[col][row];
		if(state==Constant.FLAG&&flip)
		{
			incMine();
			blockState[col][row]=Constant.UNTOUCHED;
			paintBlock(col,row,false);
		}
		else if(state==Constant.UNTOUCHED)
		{
			decMine();
			tagged=true;
			blockState[col][row]=Constant.FLAG;
			paintBlock(col,row,false);
		}
	}

	private void uncover(int col,int row)
	{
		if(!isAblock(col,row))
			return;
		if(firstUncover)//test if it is the first time to uncover a mine since restart
		{
			startTime();
			initMatric(col, row);
			firstUncover=false;
		}

		if(blockState[col][row]==Constant.UNTOUCHED || blockState[col][row]==Constant.MINE)
// the second case of the 'if' is a detail behavior of uncover9() -- two or more mines should be
// both baned.
		{
			int contain=calcValue(col,row);
			blockState[col][row]=contain;
			paintBlock(col,row,false);	
			if(contain==Constant.MINE)
			{
				die(col,row);
				return;
			}
			mineCount--;
			if(mineCount==0)
			{
				setEnabled(false);
				stopTime();
				win();
			}
			if(contain==Constant.NONE)
			{
				for(int i=0;i<9;i++)
				{
					if(i==4)
						continue;
					if(isAblock(col+i/3-1,row+i%3-1))
						uncover(col+i/3-1,row+i%3-1);
				}
				return;
			}
			return;
		}
	}
	
	private void uncover9(int col,int row)
	{
		if(!isAblock(col,row))
			return;
		startTime();
		int count=0;
		for(int i=0;i<9;i++)
		{
			if(i==4)
				continue;
			if(isAblock(col+i/3-1,row+i%3-1) && blockState[col+i/3-1][row+i%3-1]==Constant.FLAG)
				count++;
		}
		if(count==blockState[col][row])
			for(int i=0;i<9;i++)
				uncover(col+i/3-1,row+i%3-1);
	}

	private void initMatric(int x, int y)
	{
		for(int i=0;i<lc.cols;i++)
			for(int j=0;j<lc.rows;j++)
				blockContain[i][j]=Constant.NONE;
		blockContain[x][y]=Constant.MINE;
		for(int i=0;i<lc.mines;i++)
		{
			int xTemp,yTemp;
			do
			{
				xTemp=Math.abs(Constant.random.nextInt()%lc.cols);
				yTemp=Math.abs(Constant.random.nextInt()%lc.rows);
			}while(blockContain[xTemp][yTemp]==Constant.MINE);
			blockContain[xTemp][yTemp]=Constant.MINE;
		}
		blockContain[x][y]=Constant.NONE;
	}

	private int calcValue(int col,int row)
	{
		if(blockContain[col][row]==Constant.MINE)
			return Constant.MINE;
		int count=0;
		for(int i=0;i<9;i++)
			if(isAblock(col+i/3-1,row+i%3-1) && blockContain[col+i/3-1][row+i%3-1]==Constant.MINE)
				count++;
		return count;
	}
	
	private void leftDown(int x,int y)
	{
		mouseX=x;
		mouseY=y;
		leftDown=true;

		if(isAboveFace(x,y))
		{
			insideFace=true;
			fresh(pressFace());
		}
		else if(blockEnabled)
		{
			int col=getCol(mouseX),
				row=getRow(mouseY);
			faceState=Constant.OH;
			paintFace(true);
			if(!rightDown)
			{
				moveSee=true;
				pressDown(col,row);
			}
			else if(Config.canTag)
				pressDown9(col,row);
			fresh(true);
			return;
		}
	}

	private void leftUp(int x,int y)
	{
		mouseX=x;
		mouseY=y;
		leftDown=false;
		int col=getCol(x),row=getRow(y);
		if(insideFace)
		{
			if(isAboveFace(x,y))
			{
				faceState=Constant.SMILE;
				releaseFace();
				insideFace=false;
				restart();
				return;
			}
			else
			{
				insideFace=false;
				fresh(releaseFace());
			}
		}
		else if(blockEnabled)
		{
			faceState=Constant.SMILE;
			if(!rightDown&&moveSee)
				uncover(col,row);
			else if(rightDown)
			{
				releaseUp();
				uncover9(col,row);
			}
			paintFace(true);
			fresh(true);
			return;
		}
	}
	
	private void rightDown(int x,int y)
	{
		if(insideFace)
			return;
		rightDown=true;
		mouseX=x;
		mouseY=y;
		int col=getCol(mouseX),row=getRow(mouseY);

		if(!alive)
		{
			restart();
			rightDown=true;
		}
		else if(blockEnabled && Config.canTag)
		{
			if(!leftDown)
				tag(col,row,true);
			else
			{
				pressDown9(col,row);
				faceState=Constant.OH;
				paintFace(true);
			}
			fresh(true);
			return;
		}
		else
			fresh(releaseUp());
	}

	private void rightUp(int x,int y)
	{
		releaseUp();
		if(insideFace)
			return;
		fresh(true);
		rightDown=false;
	}

	public void mouseMoved(MouseEvent evt)
	{
		int x=evt.getX()-canvasX, y=evt.getY()-canvasY;
		if(insideFace)
			fresh(releaseFace());
		else
			fresh(releaseUp());
		mouseX=x;
		mouseY=y;
	}

	public void mouseDragged(MouseEvent evt)
	{
		if(!leftDown && !rightDown)
		{
			mouseMoved(evt);
			return;
		}
		int x=evt.getX()-canvasX, y=evt.getY()-canvasY;
		boolean needFresh=false;
		mouseX=x;
		mouseY=y;
		int col=getCol(mouseX),row=getRow(mouseY);

		if(insideFace && leftDown==true)
		{
			if(isAboveFace(x,y))
				fresh(pressFace());
			else
				fresh(releaseFace());
		}
		else if(blockEnabled)
		{
			if(!rightDown&&moveSee)
				fresh(pressDown(col,row));
			else if(leftDown && rightDown)
				fresh(pressDown9(col,row));
		}
	}

	private void regist(int time,int level, boolean tag)
	{
		new Register(time,level,tag);
	}

	private boolean mouseIsDown=false;
	public void mousePressed(MouseEvent evt)
	{
		int x=evt.getX()-canvasX, y=evt.getY()-canvasY;
		mineStatusChanged=true;
		if(!haveFocus)
			haveFocus=true;
		else
		{
			mouseIsDown=true;
			if( (((evt.getModifiers()&InputEvent.BUTTON3_MASK)!=0)||evt.isMetaDown())
						&&rightDown==false )
				rightDown(x,y);
			if( ((evt.getModifiers()&InputEvent.BUTTON1_MASK)!=0)
						&& leftDown==false)
				leftDown(x,y);
			if( ((evt.getModifiers()&InputEvent.BUTTON2_MASK)!=0) && leftDown==false
						&& rightDown==false)
			{
				leftDown(x, y);
				rightDown(x, y);
			}
		}
	}

	public void mouseReleased(MouseEvent evt)
	{
		int x=evt.getX()-canvasX, y=evt.getY()-canvasY;
		mineStatusChanged=true;
		if(!haveFocus)
			haveFocus=true;
		else if(!mouseIsDown)
			rightUp(x,y);
		else
		{
			mouseIsDown=false;
			if(leftDown)
				leftUp(x,y);
			if(rightDown)
				rightUp(x,y);
		}
	}
	public void mouseClicked(MouseEvent evt)
	{}
	public void mouseEntered(MouseEvent evt)
	{}
	public void mouseExited(MouseEvent evt)
	{}

	public void keyPressed(KeyEvent evt)
	{
		int key=evt.getKeyCode();
		if(key==KeyEvent.VK_F4 && ((evt.getModifiers()|InputEvent.ALT_MASK)!=0))
		{
			Constant.onlyFrame.F4Key=true;
			return;
		}
		boolean process=true;
		mineStatusChanged=true;
		if(!haveFocus)
		{
			haveFocus=true;
			process=false;
		}

		if(!process)
			return;

		switch(key)
		{
			case KeyEvent.VK_Z:
			case KeyEvent.VK_M:
				rightDown(mouseX,mouseY);
				break;
			case KeyEvent.VK_X:
			case KeyEvent.VK_N:
				leftDown(mouseX, mouseY);
				rightDown(mouseX, mouseY);
				break;
			case KeyEvent.VK_C:
			case KeyEvent.VK_B:
				leftDown(mouseX, mouseY);
				break;
		}
	}

	public void keyReleased(KeyEvent evt)
	{
		Constant.onlyFrame.F4Key=false;
		int key=evt.getKeyCode();
		boolean process=true;
		mineStatusChanged=true;
		if(!haveFocus)
		{
			haveFocus=true;
			process=false;
		}

		if(!process)
			return;

		switch(key)
		{
			case KeyEvent.VK_Z:
			case KeyEvent.VK_M:
			case KeyEvent.VK_X:
			case KeyEvent.VK_N:
			case KeyEvent.VK_C:
			case KeyEvent.VK_B:
				if(leftDown)
					leftUp(mouseX,mouseY);
				if(rightDown)
					rightUp(mouseX,mouseY);
				break;
		}
	}
	public void keyTyped(KeyEvent evt)
	{}

/*	protected void processKeyEvent(KeyEvent evt)
	{
		if(evt.getKeyCode()!=KeyEvent.VK_F4)
			super.processKeyEvent(evt);
	}*/

	private boolean haveFocus=true;
	public void focusLost(FocusEvent evt)
	{
		haveFocus=false;
	}

	public void focusGained(FocusEvent evt)
	{
		haveFocus=true;
	}
	
	public void returnFocus()
	{
		requestFocus();
	}

	public static void drawButton(Graphics g,int left,int top,int width,int height,int deepth,
								Color normal,Color bright,Color dark)
	{
		g.setColor(normal);
		g.fillRect(left+edgeSize,top+edgeSize,width,height);
		g.setColor(bright);
		for(int i=0;i<deepth;i++)
		{
			g.fillRect(left+edgeSize,top+i+edgeSize,width-i-1,1);
			g.fillRect(left+i+edgeSize,top+edgeSize,1,height-i-1);
		}
		g.setColor(dark);
		for(int i=0;i<deepth;i++)
		{
			g.fillRect(left+i+1+edgeSize,top+height-1-i+edgeSize,width-i-1,1);
			g.fillRect(left+width-1-i+edgeSize,top+i+1+edgeSize,1,height-i-1);
		}
	}

	private boolean insideMatrix(int x, int y)
	{
		return isAblock(getCol(x),getRow(y));
	}

	private boolean isAblock(int col,int row)
	{
		if(col>=0 && col<lc.cols && row>=0 && row<lc.rows)
			return true;
		else return false;
	}
	
	private boolean isAboveFace(int x,int y)
	{
		if(x>=lc.faceX && y>=Constant.faceY
				&& x<lc.faceX+Constant.faceW
				&& y<Constant.faceY+Constant.faceH)
			return true;
		else return false;
	}

	private int xAddOn=((Constant.blockX/Constant.blockW)+1)*Constant.blockW,
					yAddOn=((Constant.blockY/Constant.blockH)+1)*Constant.blockH;

	private int getCol(int x)
	{
		return (x-Constant.blockX+xAddOn)/Constant.blockW-xAddOn/Constant.blockW;
	}

	private int getRow(int y)
	{
		return (y-Constant.blockY+yAddOn)/Constant.blockH-yAddOn/Constant.blockH;
	}

	private Vector pressDownList=new Vector();
	private boolean facePressedDown=false;
	
	private boolean pressFace()
	{
		boolean freshed=releaseUp();
		if(facePressedDown==false)
		{
			paintFace(false);
			facePressedDown=true;
			return true;
		}
		return freshed;
	}
	
	private boolean releaseFace()
	{
		boolean freshed=releaseUp();
		if(facePressedDown==true)
		{
			paintFace(true);
			facePressedDown=false;
			return true;
		}
		return freshed;
	}

	private boolean pressDownNoRelease(int col, int row)
	{
		Point point;
		if(isAblock(col,row))
		{
			if(blockState[col][row]==Constant.UNTOUCHED)
			{
				if(pressDownList.indexOf(point=new Point(col, row))<0)
				{
					pressDownList.addElement(point);
					paintBlock(col,row,true);
					return true;
				}
			}
		}
		return false;
	}

	private boolean pressDown(int col, int row)
	{
		Point tempPoint;
		boolean freshed=false;
		int i=0;
		try
		{
			while(true)
			{
				tempPoint=(Point)pressDownList.elementAt(i);
				if(tempPoint.x!=col || tempPoint.y!=row)
				{
					freshed=true;
					releaseUp1(tempPoint.x,tempPoint.y);
					pressDownList.removeElementAt(i);
				}
				else
					i++;
			}
		}catch(Exception e){}
		freshed=pressDownNoRelease(col, row)||freshed;
		return freshed;
	}

	private boolean pressDown9(int col, int row)
	{
		Point tempPoint;
		int i=0;
		boolean freshed=false;
		try
		{
			while(true)
			{
				tempPoint=(Point)pressDownList.elementAt(i);
				if(tempPoint.x>col+1 || tempPoint.x<col-1 ||
						tempPoint.y>row+1 || tempPoint.y<row-1)
				{
					freshed=true;
					releaseUp1(tempPoint.x,tempPoint.y);
					pressDownList.removeElementAt(i);
				}
				else
					i++;
			}
		}catch(Exception e){}

		if(isAblock(col,row))
			for(int j=0;j<9;j++)
				freshed=pressDownNoRelease(col+j/3-1,row+j%3-1)||freshed;
		return freshed;
	}
	
	private boolean releaseUp()
	{
		Point tempPoint;
		boolean freshed=false;
		try
		{
			while(true)
			{
				tempPoint=(Point)pressDownList.firstElement();
				freshed=true;
				releaseUp1(tempPoint.x,tempPoint.y);
				pressDownList.removeElementAt(0);
			}
		}catch(Exception e){}
		return freshed;
	}

	private void releaseUp1(int col, int row)
	{
		if(isAblock(col,row))
		{
			if(blockState[col][row]==Constant.UNTOUCHED)
				paintBlock(col,row,false);
		}
	}

	public Dimension getMinimumSize()
	{
		if(Constant.isCombating)
			return Constant.screenDim;
		else
			return new Dimension(Constant.levels[Config.level].panelW, Constant.levels[Config.level].panelH);
	}

	public Dimension getPreferredSize()
	{
		return getMinimumSize();
	}
}
