/*
***********************************************************************
China Mine version 1.1
Copyright (C) 1998, 1999, 2000  Ma Guokai, from People's Republic of China

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If what you get is only part of the program, send an email to akey@fudan.edu
give the name of the program and the version number (newest by default), I'll
send you a copy of the full package.
*****************************************************************************
*/

import java.awt.*;
import java.awt.event.*;

public class MainMenu extends MenuBar
{
	public Menu gameMenu, moreMenu, helpMenu;
	public MenuItem newGame, bestTime, exitGame;
	public CheckboxMenuItem[] nameLevel;
	public CheckboxMenuItem[] styles;
	public CheckboxMenuItem[] languages;
	public MenuItem switchHotKey;
	public CheckboxMenuItem switchTag;
	public CheckboxMenuItem combat;
	public Menu selectStyleMenu;
	public Menu selectLanguageMenu;
	public MenuItem mineHelp, copyright, license, aboutMine, contributes;

	public MainMenu(MainFrame eventHandler)
	{
		gameMenu=new Menu();
		gameMenu.addActionListener(eventHandler);
		moreMenu=new Menu();
		moreMenu.addActionListener(eventHandler);
		helpMenu=new Menu();
		helpMenu.addActionListener(eventHandler);

		newGame=new MenuItem();
		newGame.addActionListener(eventHandler);
		bestTime=new MenuItem();
		bestTime.addActionListener(eventHandler);
		exitGame=new MenuItem();
		exitGame.addActionListener(eventHandler);

		switchTag=new CheckboxMenuItem();
		switchTag.addItemListener(eventHandler);
		combat=new CheckboxMenuItem();
		combat.addItemListener(eventHandler);
		nameLevel=new CheckboxMenuItem[Constant.levels.length];
		for(int i=0;i<nameLevel.length;i++)
		{
			nameLevel[i]=new CheckboxMenuItem();
			nameLevel[i].addItemListener(eventHandler);
		}
		switchHotKey=new MenuItem();
		switchHotKey.addActionListener(eventHandler);
		selectStyleMenu=new Menu();
		styles=new CheckboxMenuItem[Constant.styles.length];
		for(int i=0;i<styles.length;i++)
		{
			styles[i]=new CheckboxMenuItem();
			styles[i].addItemListener(eventHandler);
		}
		selectLanguageMenu=new Menu();
		languages=new CheckboxMenuItem[Constant.languages.length];
		for(int i=0;i<languages.length;i++)
		{
			languages[i]=new CheckboxMenuItem();
			languages[i].addItemListener(eventHandler);
		}

		mineHelp=new MenuItem();
		mineHelp.addActionListener(eventHandler);
		copyright=new MenuItem();
		copyright.addActionListener(eventHandler);
		contributes=new MenuItem();
		contributes.addActionListener(eventHandler);
		license=new MenuItem();
		license.addActionListener(eventHandler);
		aboutMine=new MenuItem();
		aboutMine.addActionListener(eventHandler);

		resetLabels();
		add(gameMenu);
		{
			gameMenu.add(newGame);
			gameMenu.addSeparator();
			for(int i=0;i<nameLevel.length;i++)
			{
				gameMenu.add(nameLevel[i]);
				nameLevel[i].setState(false);
			}
			nameLevel[Config.level].setState(true);
			gameMenu.addSeparator();
			gameMenu.add(switchTag);
			switchTag.setState(Config.canTag);
			gameMenu.add(combat);
			combat.setEnabled(!Constant.isApplet || Constant.mineCGIstring!=null);
			combat.setState(false);
			gameMenu.addSeparator();
			gameMenu.add(bestTime);
			bestTime.setEnabled(!Constant.isApplet || Constant.mineCGIstring!=null);
			gameMenu.addSeparator();
			gameMenu.add(exitGame);
		}
		add(moreMenu);
		{
			moreMenu.add(switchHotKey);
			moreMenu.addSeparator();
			moreMenu.add(selectStyleMenu);
			{
				for(int i=0;i<styles.length;i++)
				{
					selectStyleMenu.add(styles[i]);
					styles[i].setState(false);
				}
				styles[Config.style].setState(true);
			}
			moreMenu.addSeparator();
			moreMenu.add(selectLanguageMenu);
			{
				for(int i=0;i<languages.length;i++)
				{
					selectLanguageMenu.add(languages[i]);
					languages[i].setState(false);
				}
				languages[Config.language].setState(true);
			}
		}
		add(helpMenu);
		setHelpMenu(helpMenu);
		{
			helpMenu.add(mineHelp);
			helpMenu.addSeparator();
			helpMenu.add(copyright);
			helpMenu.add(license);
			license.setEnabled(Constant.isApplet);
			helpMenu.addSeparator();
			helpMenu.add(contributes);
			helpMenu.add(aboutMine);
		}
	}

	public void resetLabels()
	{
		gameMenu.setLabel(Constant.languages[Config.language].menuGame);
		moreMenu.setLabel(Constant.languages[Config.language].menuMore);
		helpMenu.setLabel(Constant.languages[Config.language].menuHelp);

		newGame.setLabel(Constant.languages[Config.language].menuNew);
		bestTime.setLabel(Constant.languages[Config.language].menuBest);
		exitGame.setLabel(Constant.languages[Config.language].menuExit);

		for(int i=0;i<nameLevel.length;i++)
			nameLevel[i].setLabel(Constant.levels[i].levelNameFirstUpperCased);
		switchTag.setLabel(Constant.languages[Config.language].menuTag);
		combat.setLabel(Constant.languages[Config.language].combat);
		switchHotKey.setLabel(Config.hotKeyEnabled?Constant.languages[Config.language].menuDisable:
				Constant.languages[Config.language].menuEnable);
		selectStyleMenu.setLabel(Constant.languages[Config.language].menuStyle);
		for(int i=0;i<styles.length;i++)
			styles[i].setLabel(Constant.styles[i].styleName);
		selectLanguageMenu.setLabel(Constant.languages[Config.language].menuLanguage);
		for(int i=0;i<languages.length;i++)
			languages[i].setLabel(Constant.languages[i].languageName);

		mineHelp.setLabel(Constant.languages[Config.language].menuHowTo);
		copyright.setLabel(Constant.languages[Config.language].menuCopyright);
		license.setLabel(Constant.languages[Config.language].menuLicense);
		contributes.setLabel(Constant.languages[Config.language].contributes);
		aboutMine.setLabel(Constant.languages[Config.language].menuAbout);
	}
}